From 13995c11064f42e4440e9ded11198faaf5e6b7bc Mon Sep 17 00:00:00 2001
From: Masahiko Sawada <sawada.mshk@gmail.com>
Date: Fri, 13 Dec 2024 16:54:46 -0800
Subject: [PATCH v7 6/8] radixtree.h: Add RT_NUM_KEY API to get the number of
 keys.

Author:
Reviewed-by:
Discussion: https://postgr.es/m/
Backpatch-through:
---
 src/include/lib/radixtree.h | 13 +++++++++++++
 1 file changed, 13 insertions(+)

diff --git a/src/include/lib/radixtree.h b/src/include/lib/radixtree.h
index c8efa61ac7c..4b5598a8104 100644
--- a/src/include/lib/radixtree.h
+++ b/src/include/lib/radixtree.h
@@ -126,6 +126,7 @@
  * RT_ITERATE_NEXT	- Return next key-value pair, if any
  * RT_END_ITERATE	- End iteration
  * RT_MEMORY_USAGE	- Get the memory as measured by space in memory context blocks
+ * RT_NUM_KEYS		- Get the number of key-value pairs in radix tree
  *
  * Interface for Shared Memory
  * ---------
@@ -197,6 +198,7 @@
 #define RT_DELETE RT_MAKE_NAME(delete)
 #endif
 #define RT_MEMORY_USAGE RT_MAKE_NAME(memory_usage)
+#define RT_NUM_KEYS RT_MAKE_NAME(num_keys)
 #define RT_DUMP_NODE RT_MAKE_NAME(dump_node)
 #define RT_STATS RT_MAKE_NAME(stats)
 
@@ -313,6 +315,7 @@ RT_SCOPE	RT_VALUE_TYPE *RT_ITERATE_NEXT(RT_ITER * iter, uint64 *key_p);
 RT_SCOPE void RT_END_ITERATE(RT_ITER * iter);
 
 RT_SCOPE uint64 RT_MEMORY_USAGE(RT_RADIX_TREE * tree);
+RT_SCOPE int64 RT_NUM_KEYS(RT_RADIX_TREE * tree);
 
 #ifdef RT_DEBUG
 RT_SCOPE void RT_STATS(RT_RADIX_TREE * tree);
@@ -2807,6 +2810,15 @@ RT_MEMORY_USAGE(RT_RADIX_TREE * tree)
 	return total;
 }
 
+RT_SCOPE int64
+RT_NUM_KEYS(RT_RADIX_TREE * tree)
+{
+#ifdef RT_SHMEM
+	Assert(tree->ctl->magic == RT_RADIX_TREE_MAGIC);
+#endif
+	return tree->ctl->num_keys;
+}
+
 /*
  * Perform some sanity checks on the given node.
  */
@@ -3130,6 +3142,7 @@ RT_DUMP_NODE(RT_NODE * node)
 #undef RT_END_ITERATE
 #undef RT_DELETE
 #undef RT_MEMORY_USAGE
+#undef RT_NUM_KEYS
 #undef RT_DUMP_NODE
 #undef RT_STATS
 
-- 
2.43.5

